use serde::Deserialize;

use std::fmt;
use std::fs;

pub fn read_toml_from_file<T>(path: &String) -> Result<T,TomlError>
where T: for<'de> Deserialize<'de> {
	let text = match fs::read_to_string(path) {
		Ok(t) => t,
		Err(e) => {
			return Err(TomlError::FileError{
				path: path.clone(),
				io_error: e
			});
		}
	};
	match toml::from_str(&text) {
		Ok(t) => Ok(t),
		Err(e) => {
			return Err(TomlError::ParseError{
				path: path.clone(),
				toml_error: e
			});
		}
	}
}

pub enum TomlError {
	FileError{ path: String, io_error: std::io::Error },
	ParseError{ path: String, toml_error: toml::de::Error },
}

impl fmt::Display for TomlError {
	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
		match self {
			Self::FileError{path, io_error} =>
				write!(f,"Error while reading file '{path}': {io_error}"),
			Self::ParseError{path, toml_error} =>
				write!(f,"Unable to parse file as toml '{path}':\n{toml_error}"),
		}
	}
}
